#include <FL/Fl.h>
#include "TargaImage.h"
#include "ImageWidget.h"
#include <FL/fl_file_chooser.h>
#include <FL/Fl_Window.h>
#include <FL/Fl_Button.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>


// Static variables have scope only in the current file.
static ImageWidget  *image_widget = 0; // The widget that displays the image
static TargaImage   *current_image = 0;// The current image object

static Fl_Button    *file_button;


static int
Load_Function(char *filename)
{
    TargaImage	*new_image;

    fprintf(stderr, "Loading %s...\n", filename);

    new_image = TargaImage::Load_Image(filename);
    if ( new_image == NULL )
    {
	fprintf(stderr, "Error loading image from file %s\n", filename);
	return 0;
    }

    // Free the memory from the current image
    delete current_image;

    // Replace the current image
    current_image = new_image;
    image_widget->Set_Image(current_image);

    return 1;
}


static void
File_Button_Callback(Fl_Widget *w, void *d)
{
    char    *fname;
    if ( ( fname = fl_file_chooser("Choose a TGA file", "*.tga", NULL) ) )
	Load_Function(fname);
}


// Argument processing callback. Does nothing at this point.
static int
Arg_Callback(int argc, char *argv[], int &i)
{
    return 0;
}

int
main(int argc, char *argv[])
{
    Fl_Window	window(810, 435, "CS559 Project 1 Skeleton");
    int		script_arg;

    // Do argument processing. At the end of this, script_arg contains
    // the first non-switch argument, which if not 0 or argc is the
    // location of the script file name in the argument list.
    Fl::args(argc, argv, script_arg, Arg_Callback);
    if ( script_arg == 0 )
    {
	fprintf(stderr, "Error: Unrecognised argument.\n");
	return 1;
    }

    Fl::visual(FL_RGB);

    window.begin();

    file_button = new Fl_Button(5, 5, 100, 20, "File");
    file_button->callback(File_Button_Callback, 0);

    // We use a separate image containing widget so that other widgets, like
    // buttons, can be added to the main window. This creates that image
    // display widget, and then sets it as the thing to be resized when the
    // window is resized. This allows us to resize thing properly when the
    // image size changes.
    image_widget = new ImageWidget(5, 30, 400, 400, "Image");
    window.resizable(image_widget);

    window.end();

    if ( script_arg < argc )
    {
	if ( ! Load_Function(argv[script_arg]) )
	{
	    return 1;
	}
    }

    window.show(argc, argv);

    return Fl::run();
}


